//Java-imports
import javax.swing.*;
import java.awt.*;
import java.net.*;
import java.io.*;

/**
 * <strong>Program:</strong>      BildeBok<br>
 * <strong>File:</strong>         AboutBox.java<br>
 * <strong>Description:</strong>  Klasse som viser en about-box for programmet<br>
 * <strong>Author:</strong>       Kenneth Stigen - kenneths@frodo.hiof.no<br>
 * <strong>Environment:</strong>  Windows - jdk 1.3.1<br>
 * <strong>Notes:</strong>        Dette er en del av prosjektet i grafisk databehandling V2002<br>
 *<br>
 * @author Kenneth Stigen (mailto: kenneths@frodo.hiof.no, web: http://frodo.hiof.no/~kenneths)
 * @version Version 1.0
 */
public class AboutBox extends JFrame{
   
   //PRIVATE VARIABLES
   Container c;
   
   /**
    * Constructor
    * @param type Hvilken type box som skal vises
    *
    * Denne sjekker p om det skal vre en hjelpebox
    * eller om det skal vre en aboutbox som skal vises.
    */
   public AboutBox(String type){
      super("Bildebok - Kenneth Stigen");
      addWindowListener(new CloseWindow());
      if(type.equals("about")){
	 setSize(400,400);
	 setupBoth();
	 setupAbout();
	 show();
      }else if(type.equals("help")){
	 setSize(400,400);
	 setupBoth();
	 setupHelp();
	 show();
      }
   }
   
   /**
    * setupBoth
    *
    * Metode som bestemmer hvor p skjermen boksen skal dukke opp.
    * Denne setter ogs opp en container som objektene kan legges p.
    */
   private void setupBoth(){
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      
      Dimension frameSize = this.getSize();
      if (frameSize.height > screenSize.height)
	 frameSize.height = screenSize.height;
      if (frameSize.width > screenSize.width)
	 frameSize.width = screenSize.width;
      this.setLocation ((screenSize.width - frameSize.width)/2,(screenSize.height - frameSize.height)/2);
      
      c = getContentPane();
      c.setLayout(null);
      c.setBackground(new Color(229,242,255));
   }
   
   /**
    * setupHelp
    *
    * Metode som setter opp GUI for en hjelp boks.
    * Den viser en html-side som beskriver hjelpen.
    */
   private void setupHelp(){
      JLabel help1;
      JTextArea help2;
      JEditorPane htmlShower;
      JScrollPane htmlScroller;
      String h1 = "Hjelp - Bildebok";
      
      help1 = new JLabel(h1);
      help1.setBounds(100, 10, 360, 25);
      help1.setFont(new Font("Arial", Font.BOLD, 24));
      c.add(help1);
      
      try{
	 File f = new File("help\\help.html");
         URL helpSite = f.toURL();
         htmlShower = new JEditorPane(helpSite);
         htmlShower.setEditable(false);
	 
         htmlScroller = new JScrollPane(htmlShower);
         htmlScroller.setBounds(10,40,375,325);
         c.add(htmlScroller);
      }catch(Exception ex){
         ex.printStackTrace();
      }
   }
   
   /**
    * setupAbout
    *
    * Metode som setter opp GUI til aboutboksen.
    * Den oppretter et glcanvas, lager thread og setter
    * i gang animasjon av glcanvaset.
    */
   private void setupAbout(){
         
      AboutCanvas canv;
      Thread t;
      JLabel head1, head2, head3, head4;
      
      String head1Text = "Prosjekt i GrafDat";
      String head2Text = "Utviklet ved HI vren 2002";
      String head3Text = "Kenneth Stigen";
      String head4Text = "kenneths@frodo.hiof.no";
      
      head1 = new JLabel(head1Text);
      head1.setFont(new Font("Arial", Font.BOLD, 24));
      head1.setBounds(82,10,360,25);
      c.add(head1);
      
      head2 = new JLabel(head2Text);
      head2.setBounds(110,40,360,25);
      c.add(head2);
      
      head3 = new JLabel(head3Text);
      head3.setBounds(150,245,360,25);
      c.add(head3);
      
      head4 = new JLabel(head4Text);
      head4.setBounds(130,275,360,25);
      c.add(head4);
      
      
      canv = new AboutCanvas( 30,30);
      canv.setBounds(130,105,130,130);
      c.add(canv);
      t = new Thread(canv);
      t.start();
   }
}
