//gl4java - imports
import gl4java.GLContext;
import gl4java.awt.GLCanvas;
import gl4java.utils.glut.*;
import gl4java.awt.GLAnimCanvas;
import gl4java.utils.textures.*;

/**
 * <strong>Program:</strong>      BildeBok<br>
 * <strong>File:</strong>         AboutCanvas.java<br>
 * <strong>Description:</strong>  openGL - klasse som viser en teapot som roterer<br>
 * <strong>Author:</strong>       Kenneth Stigen - kenneths@frodo.hiof.no<br>
 * <strong>Environment:</strong>  Windows - gl4java<br>
 * <strong>Notes:</strong>        Dette er en del av prosjektet i grafisk databehandling V2002<br>
 *
 * @author Kenneth Stigen (mailto: kenneths@frodo.hiof.no, web: http://frodo.hiof.no/~kenneths)
 * @version Version 1.0
 */
public class AboutCanvas extends GLCanvas implements Runnable{
   
   //PRIVATE VARIABLES
   private FargeGen myColor;
   private GLUTFunc glut = null;
   private float deg = 0.0f;
   
   /**
    * Constructor
    * @param width Integer verdien som bestemmer hvor vidt canvaset er.
    * @param height Integer verdien som bestemmer hvod hyt canvaset er.
    *
    * Denne setter width, og height
    * Oppretter ogs objekt for  sette farge.
    */
   public AboutCanvas(int width, int height){
      super(width, height);
      myColor = new FargeGen(this);
   }
   
   /**
    * preInit
    *
    * Metode som setter verdier som glcanvaset trenger.
    * Metoden blir kalt nr canvaset opprettes
    * Sier ifra at doubleBuffering er p, og at stereoView er av.
    * accumSize er satt til 8 for  ke kvaliteten p scenen.
    */
   public void preInit(){
      doubleBuffer = true;
      stereoView = false;
      accumSize = 8;
   }
   
   /**
    * init
    *
    * Setter forskjellige startverdier for glcanvaset.
    */
   public void init(){
      
      //Enable Texture Mapping
      gl.glEnable(GL_TEXTURE_2D);
      
      //Oppretter glut objektet.
      glut = new GLUTFuncLightImpl(gl, glu);
      reshape(getSize().width, getSize().height);
      
      //Bestemmer verdiene for lyset
      float ambient[] = {0.5f,0.5f,0.5f,1.0f };
      float diffuse[] = {1.0f,1.0f,1.0f,1.0f };
      float position[] = {0.0f,1.0f,-1.0f,0.0f };
      float lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };
      
      //Lar gl objektet f vite om lyset
      gl.glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
      gl.glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
      gl.glLightfv(GL_LIGHT0, GL_POSITION, position);
      gl.glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,2);
      
      //Enabler lys i gl.
      gl.glEnable(GL_LIGHT0);
      gl.glEnable(GL_LIGHTING);
      
      //Gjr tegningen glattere
      gl.glShadeModel(GL_SMOOTH);
      
      //"Enabler" dybdesortering
      gl.glEnable(GL_DEPTH_TEST);
      gl.glDepthFunc(GL_LESS);
      
      //GJR KLAR FOR BEZIER
      gl.glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
      gl.glEnable(GL_MAP2_VERTEX_3);
      gl.glEnable(GL_AUTO_NORMAL);
      gl.glEnable(GL_NORMALIZE);
      
      //Setter bakgrunnsfarge
      gl.glClearColor(0.9f, 0.95f, 1.0f, 1.0f);
      
      
   }
   
   
   /**
    * display
    *
    * Metode som visker klart canvaset, kaller p draw-metoden.
    */
   public void display(){
      //Srger for at dette tegneobjektet blir det som er current.
      if (glj.gljMakeCurrent() == false){
	 return;
      }
      
      gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
      gl.glClear(GL_ACCUM_BUFFER_BIT);
      gl.glLoadIdentity();
      
      
      gl.glPushMatrix();
      draw();
      gl.glPopMatrix();
      
      glj.gljSwap();
      glj.gljFree();
      
      
   }
   
   /**
    * draw()
    * Metode som setter fargen til  vre bl gjennom myColor.
    * Roterer s mange grader som deg er satt til.
    * Variabelen deg forandres etter hvert som lkken i run-metoden
    * gr.
    * Tegner deretter ut den tradisjonelle teapot.
    */
   private void draw(){
      myColor.setBlueMaterial();
      gl.glRotatef(deg, 1.0f, 1.0f, 1.0f);
      glut.glutSolidTeapot(0.5);
   }
   
   /**
    * run
    *
    * Metode som trengs for  kunne bruke denne klassen som en thread
    * Metode starter en lkke som er evig. ker roteringsvariabelen deg
    * med 3.0 for hver runde, og repainter. Metoden "sover" i 30 millisekunder
    * for hver gang canvaset tegnes p nytt.
    */
   public void run(){
      while(true){
	 deg += 3.0f;
	 repaint();
	 try{
	    Thread.sleep(30);
	 }catch(Exception ex){
	    ex.printStackTrace();
	 }
      }
   }
}
