//Gl4java imports
import gl4java.awt.GLCanvas;
import gl4java.utils.glut.*;
import gl4java.utils.textures.*;

//Java imports
import java.util.Vector;
import java.awt.event.*;

/**
 * <strong>Program:</strong>      BildeBok<br>
 * <strong>File:</strong>         BildeBokCanvas.java<br>
 * <strong>Description:</strong>  GLCanvaset som tar seg av all uttegning av openGL objekter<br>
 * <strong>Author:</strong>       Kenneth Stigen - kenneths@frodo.hiof.no<br>
 * <strong>Environment:</strong>  Windows - gl4java<br>
 * <strong>Notes:</strong>        Dette er en del av prosjektet i grafisk databehandling V2002<br>
 *
 * @author Kenneth Stigen (mailto: kenneths@frodo.hiof.no, web: http://frodo.hiof.no/~kenneths)
 * @version Version 1.0
 */
public class BildeBokCanvas extends GLCanvas implements Runnable, KeyListener{
   
   private String[] images = {"images\\forside_redigert.png",
			      "images\\bok1.png",
			      "images\\side1_redigert_2.png",
			      "images\\side2_redigert.png",
			      "images\\side3_redigert.png",
			      "images\\side4_redigert.png",
			      "images\\side5_redigert.png",
			      "images\\side6_redigert.png",
			      "images\\sverige.png"};
   
   private int page = 2;
   
   //Variabler brukt i blaingen av boken.
   private float lastPageMidInnZ = 0.055f;
   private float lastPageMidOutZ = 0.055f;
   private float lastPageOutZ = 0.055f;
   
   //Den lokale faktoren for minsking av hyre side
   private float decFactLocal = 0.03f;
   
   private boolean showPrev = false;
   private boolean showNext = false;
   private int first = 0;
   private boolean toOpen = true;
   private boolean debugOn = false;
   
   private BildeBokFrame myParent;
   
   private TexturLaster myTextureLoader;
   private Perm myPerm;
   private PagePainter myPage;
   
   //antall grader for pning av bok
   private float deg = 0.0f;
   
   //Variabel for rotering av ryggen.
   private float xTranslate = 0.0f;
   
   //Variabler for zooming
   private double naviX = 0.0;
   private double naviY = 0.0;
   private double naviZ = 10.0;
   
   //GL-variabel
   private GLUTFunc glut = null;
   
   //Farge variabel
   private FargeGen myColor;
   
   /**
    * Constructor
    * @param myParent Frame objektet som opprettet dette objektet
    * @param width Vidden p dette canvaset
    * @param height Hyden p dette canvaset
    *
    * Constructoren setter strrelse p canvaset
    * I tillegg oppretter den instanser av TexturLaster, Perm og PagePainter
    * Legger ogs til en keylistener til canvaset, for  fange opp zooming.
    */
   public BildeBokCanvas(BildeBokFrame myParent, int width, int height){
      super(width, height);
      debug("Constructor");
      this.myParent = myParent;
      myColor = new FargeGen(this);
      myTextureLoader = new TexturLaster(this);
      myPerm = new Perm();
      myPage = new PagePainter();
      this.addKeyListener(this);
   }
   
   /**
    * preInit
    *
    * Metode som setter verdier som glcanvaset trenger.
    * Metoden blir kalt nr canvaset opprettes
    * Sier ifra at doubleBuffering er p, og at stereoView er av.
    * accumSize er satt til 8 for  ke kvaliteten p scenen.
    */
   public void preInit(){
      doubleBuffer = true;
      stereoView = false;
      accumSize = 8;
   }
   
   /**
    * init
    *
    * Setter forskjellige startverdier for glcanvaset.
    */
   public void init(){
      //Enable Texture Mapping
      gl.glEnable(GL_TEXTURE_2D);
      
      //Oppretter glut objektet.
      glut = new GLUTFuncLightImpl(gl, glu);
      reshape(getSize().width, getSize().height);
      
      //Bestemmer verdiene for lyset
      float ambient[] = {0.5f,0.5f,0.5f,1.0f };
      float diffuse[] = {1.0f,1.0f,1.0f,1.0f };
      float position[] = {0.0f,1.0f,-1.0f,0.0f };
      float lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };
      
      //Lar gl objektet f vite om lyset
      gl.glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
      gl.glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
      gl.glLightfv(GL_LIGHT0, GL_POSITION, position);
      gl.glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,2);
      
      //Enabler lys i gl.
      gl.glEnable(GL_LIGHT0);
      gl.glEnable(GL_LIGHTING);
      
      //Gjr tegningen glattere
      gl.glShadeModel(GL_SMOOTH);
      
      //"Enabler" dybdesortering
      gl.glEnable(GL_DEPTH_TEST);
      gl.glDepthFunc(GL_LESS);
      
      //GJR KLAR FOR BEZIER
      gl.glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
      gl.glEnable(GL_MAP2_VERTEX_3);
      gl.glEnable(GL_AUTO_NORMAL);
      gl.glEnable(GL_NORMALIZE);
      
      //Gjr klar for texture
      gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
      
      //Laster alle texturene
      myTextureLoader.LoadTextures(images);
      
      //Setter bakgrunnsfarge
      gl.glClearColor(0.9f, 0.95f, 1.0f, 1.0f);
      
   }
   
   /**
    * display()
    *
    * Visker klart canvaset, og setter posisjon til yet.
    * Roterer ogs s mye som er blitt spesifisert gjennom
    * musebevegelsene til brukeren.
    * Kaller til slutt p drawBook, som tegner ut boken.
    */
   public void display(){
      //Srger for at dette tegneobjektet blir det som er current.
      if (glj.gljMakeCurrent() == false){
	 return;
      }
      
      gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
      gl.glClear(GL_ACCUM_BUFFER_BIT);
      
      //Laster identitetsmatrisen
      gl.glLoadIdentity();
            
      // take it away
      //gl.glTranslatef(0.0f,0.0f,-10.0f);
      glu.gluLookAt(naviX, naviY, naviZ, 0.0, 0.0, -100.0, 0.0, 1.0, 0.0);
      // rotate around x-axis
      gl.glRotatef(myParent.getRotX(),1.0f,0.0f,0.0f);
      // rotate around z-axis
      gl.glRotatef(myParent.getRotZ(),0.0f,0.0f,1.0f);
      //gl.glTranslatef(0.0f,0.0f,-6.0f);
      //drawLines();
      
      gl.glPushMatrix();
      drawBook();
      gl.glPopMatrix();
      
      gl.glPushMatrix();
      //drawLines();
      gl.glPopMatrix();
            
      glj.gljSwap();
      glj.gljFree();
   }
   
   /**
    * drawTopOfBook
    * @param texture Den boolske verdien som sier om textur skal legges p eller ikke
    *
    * Metoden tegner ut forsiden av boken.
    */
   public void drawTopOfBook(boolean texture){
      gl.glPushMatrix();
      if(texture){
	 myColor.setWhiteMaterial();
	 gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	 myTextureLoader.bind(0);
	 //myTextureLoader.LoadGLTextures(images[0]);
	 gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, myPerm.getTxpts());
	 
      }
      
      //Tegner ut verste del av permen med tekstur.
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,2,0.0f,1.0f,6,4, myPerm.getTopPerm());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      //Disabler teksturen og setter tilbake den lysebl fargen.
      if(texture){
	 gl.glDisable(GL_MAP2_TEXTURE_COORD_2);
	 myColor.setLightBlueMaterial();
      }
      
      //Tegner bindeleddene mellom de to delene av forsiden.
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,2,0.0f,1.0f,6,3, myPerm.getTopPermFrontSide());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,2,0.0f,1.0f,6,3, myPerm.getTopPermBackSide());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      //Translater 0.05 bakover, og tegner indre del av forsiden
      gl.glTranslatef(0.0f, 0.0f, -0.05f);
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,2,0.0f,1.0f,6,4, myPerm.getTopPerm());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      gl.glPopMatrix();
      
   }
   
   /**
    * drawBack
    *
    * Metode som tegner ytre perm.
    * Flytter seg xTranslate verdier til hyre p x-aksen
    * Tegner deretter indre perm.
    */
   private void drawBack(){
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPerm.getBackPerm());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      gl.glPushMatrix();
      gl.glTranslatef(xTranslate, 0.0f, 0.0f);
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPerm.getBackPermInner());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      gl.glPopMatrix();
   }
   
   /**
    * drawBackCurl
    *
    * Metode som tegner bindeleddet mellom indre og ytre perm.
    * Tegner frst p toppen av boken, deretter p bnn av boken.
    */
   private void drawBackCurl(){
      
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,4, myPerm.getBackPermCurl());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,4, myPerm.getBackPermCurlBot());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
   }
   
   /**
    * drawBotCurl
    *
    * Metode som tegner den lille byen p boken. Denne
    * metoden tegner byen p baksiden av boken.
    * Tegner frst bindeleddene, deretter curl'ene med et mellomrom
    * p 0.05
    */
   private void drawBotCurl(){
      gl.glPushMatrix();
      gl.glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
      gl.glTranslatef(-1.1f, 0.0f, -0.45f);
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,3, myPerm.getCurlSide());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      gl.glRotatef(180.0f, 0.0f, 0.0f, 1.0f);
      gl.glTranslatef(-1.1f, 0.0f, 0.0f);
      
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,3, myPerm.getCurlSide());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      gl.glPopMatrix();
      
      gl.glPushMatrix();
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,4, myPerm.getBotCurl());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      gl.glTranslatef(0.0f, 0.0f, -0.05f);
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,4, myPerm.getBotCurl());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      gl.glPopMatrix();
   }
   
   /**
    * drawTopCurl
    *
    * Metode som tegner den lille byen p boken. Denne
    * metoden tegner byen p forsiden av boken.
    * Tegner frst bindeleddene, deretter curl'ene med et mellomrom
    * p 0.05
    */
   private void drawTopCurl(){
      gl.glPushMatrix();
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,3, myPerm.getCurlSide());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      gl.glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
      gl.glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
      gl.glTranslatef(-1.1f, 0.0f, 0.0f);
      
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,3, myPerm.getCurlSide());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      gl.glPopMatrix();
      
      gl.glPushMatrix();
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,4, myPerm.getTopCurl());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      
      gl.glTranslatef(0.0f, 0.0f, -0.05f);
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,4, myPerm.getTopCurl());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      gl.glPopMatrix();
   }
   
   /**
    * drawTopOuter
    *
    * Metode som tegner bindeleddet mellom permen p ytterkantene.
    */
   private void drawTopOuter(){
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,3,0.0f,1.0f,9,3, myPerm.getTopOuter());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
   }
   
   /**
    * openBook
    *
    * Metode som pner boken.
    */
   public void openBook(){
      
      //De forskjellige radiene som brukes for  beregne sirkelverdiene
      //til F-verdiene.
      double pagesRadUp = 0.5; //verste radius
      double pagesRadMid2 = 0.35; //verste midtre radius
      double pagesRadMid1 = 0.2; //Nederste midtre radius
      double pagesRadDown = 0.05f; //Nederste radius
      
      //Variabler som brukes til  sette verdiene p innholdet av boken, for 
      //f den lille byen nr boken er pnet.
      float xFUp = 0.0f;
      float zFUp = 0.0f;
      float xFMid2 = 0.0f;
      float zFMid2 = 0.0f;
      float xFMid1 = 0.0f;
      float zFMid1 = 0.0f;
      float xFDown = 0.0f;
      float zFDown = 0.0f;
      
      //Variabel for  f det visuelle s bra som mulig. Legges til zVerdiene til BotPages
      float toAdd = 0.0003f;
      
      //Verdier som brukes for  flytte xverdiene til sidene, for  f den skrningen
      //p innholdet p hyre side.
      float toMoveUp = (float)(pagesRadUp * Math.cos((Math.PI/2) * (double)((90)))) ;
      float toMoveMid1 = (float)(pagesRadMid2 * Math.cos((Math.PI/2) * (double)((90)))) ;
      float toMoveMid2 = (float)(pagesRadMid1 * Math.cos((Math.PI/2) * (double)((90)))) ;
      float toMoveDown = (float)(pagesRadDown * Math.cos((Math.PI/2) * (double)((90)))) ;
      
      //Lkke som gr fra 0 til 90. Ryggen skal roteres 90 grader.
      for(double i = 0.0; i < 90.0; i++){
	 
	 //Verdier som brukes til  flytte innholdet i boken i samsvar som permen pnes.
	 //(Math.PI/2) tilsvarer 90 grader
	 //Grunnen til at jeg legger til 90 p i (double)((i+90)/90) er at jeg vil at
	 //Roteringen skal begynne p toppen og g ned mot venstre
	 //Jeg deler p 90 for lkken gr 90 ganger.
	 //Z-verdiene trekkes fra 0.055 for  kompensere p forskjellene fra punktet som
	 //det roteres om.
	 xFUp = (float)(pagesRadUp * Math.cos((Math.PI/2) * (double)((i+90)/90))) ;
	 zFUp = (float)(pagesRadUp * Math.sin((Math.PI/2)* (double)((i+90)/90))) - 0.055f;
	 
	 xFMid2 = (float)(pagesRadMid2 * Math.cos((Math.PI/2) * (double)((i+90)/90))) ;
	 zFMid2 = (float)(pagesRadMid2 * Math.sin((Math.PI/2) * (double)((i+90)/90))) - 0.055f;
	 
	 xFMid1 = (float)(pagesRadMid1 * Math.cos((Math.PI/2) * (double)((i+90)/90))) ;
	 zFMid1 = (float)(pagesRadMid1 * Math.sin((Math.PI/2) * (double)((i+90)/90))) - 0.055f;
	 
	 xFDown = (float)(pagesRadDown * Math.cos((Math.PI/2) * (double)((i+90)/90))) ;
	 zFDown = (float)(pagesRadDown * Math.sin((Math.PI/2) * (double)((i+90)/90))) - 0.055f;
	 
	 //Roteringsvariabelen forandres. Slik at til slutt vil ryggen bli rotert 90 grader
	 deg += 1.0f;
	 
	 //Verdie som kes for  flytte den indre delen av ryggen litt hyere opp for at mellomrommet
	 //mellom ytre og indre rygg skal se lik ut som den gjorde fr rotering.
	 //Den skal flyttes totalt 0.05 p x-aksen -> 0.05/90
	 xTranslate += (0.05f/90);
	 
	 //Setter verdiene til permklassen
	 myPerm.setBackPermInnerValues((-0.05f/90), (0.05f/90), (-0.05f/90), (-0.05f/90));
	 myPerm.setBackPermCurl((0.03f/90), (0.05f/90), (-0.025f/90), (0.03f/90), (-0.05f/90), (0.05f/90));
	 
	 //Setter verdiene som skal forandres til PagePainter.
	 myPage.setBotPages( (xFDown), (xFMid1), (xFMid2), (xFUp), (zFDown + toAdd), (zFMid1 + toAdd), (zFMid2 + toAdd), (zFUp + toAdd) );
	 
	 //X-verdiene som skal flyttes mot venstre for  f skrningen p sidene
	 myPage.addXValues(toMoveUp/90, toMoveMid1/90, toMoveMid2/90, toMoveDown/90);
	 
	 //Gjr klar fo  vise frste siden av innholdet
	 myPage.updateTextPage();
	 
	 //Kompensasjon for ryggen av boken.
	 myPage.addRyggPage(0.0003f);
	 
	 toAdd += 0.0003f;
	 
	 repaint();
	 try{
	    Thread.sleep(17);
	 }catch(Exception ex){
	    ex.printStackTrace();
	 }
      }
      
      //Setter egenskapene p knappene i framen.
      myParent.enableAnim(false);
      myParent.enableTurn(true);
   }
   
   /**
    * drawBook
    *
    * Metode som faktisk tegner ut hele boken med innhold
    */
   public void drawBook(){
      myColor.setLightBlueMaterial();
      gl.glPushMatrix();
      gl.glTranslatef(0.5f, -2.0f,-0.05f);
      gl.glRotatef(deg, 0.0f, -1.0f, 0.0f);  //Roteringen som brukes nr boken pnes
      gl.glTranslatef( -0.5f, 2.0f, 0.05f);
      
      gl.glPushMatrix();
      gl.glTranslatef(0.5f, -2.0f,0.5f);
      gl.glRotatef(deg, 0.0f, -1.0f, 0.0f); //Dobbel rotering siden forsiden av boken skal roteres 180 grader.
      gl.glTranslatef( -0.5f, 2.0f, -0.5f);
      drawTopOfBook(true);
      drawTopCurl();
      drawTopOuter();
      gl.glPopMatrix();
      
      drawBack();
      drawBackCurl();
      drawLeftKant();
      gl.glPopMatrix();
      
      gl.glPushMatrix();
      gl.glTranslatef(0.0f, 0.0f, -0.5f);
      drawTopOfBook(false);
      drawTopOuter();
      gl.glPopMatrix();
      drawBotCurl();
      drawKanter(true, true);
      
      
      
   }
   
   /**
    * turningProcess
    *
    * Metode som tar seg av all jobben med  bla en side, og gjr klar
    * neste side, og tegne ut riktig p venstre side.
    */
   private void turningProcess(){
      
      //Sikkerhet for at brukeren ikke skal trykke turnPage mens en annen side holder
      // p  snu.
      myParent.enableTurn(false);
      if(first <= 0){
	 myPage.decreaseEdge(); //Minsker tykkelsen p hyre side
	 myPage.makeNext(); //Lager klar siden som ligger bak den som skal blas
	 showNext = true; //Sier at neste side skal kunne vises
	 turnPage(); //Kaller p metoden som faktisk blar den aktive siden
	 myPage.rotatePages(); //"Roterer" sidene, gjr klar til ny blaing
	 myPage.initLeftEdge(); //Initierer venstre side for  bli bygget p
	 repaint();
	 showPrev = true;	//Vi skal tegne ut venstre side ogs.
      }else{
	 myPage.decreaseEdge();
	 myPage.makeNext();
	 showNext = true;
	 turnPage();
	 myPage.rotatePages();
	 myPage.increaseLeftEdge(); //Vi ker venstre side i stedet for  initiere den.
	 repaint();
      }
      
      // Logikk for  vite hvilken textur som skal legges p.
      if(page+1 > images.length-1){
	 page = 2;
      }else{
	 page++;
      }
      myParent.enableTurn(true); //Gjr knappen turnPage enablet igjen.
   }
   
   
   
   
   private void turnPage(){
      float decFact = 0.03f;
      
      //Radiusene til de forskjellige punktene p siden
      //ingen radius for det innerste punktet da dette str i ro.
      double outerRadius = myPage.getOutX();
      double midOutRadius = myPage.getMidOutX();
      double midInnRadius = myPage.getMidInnX();
      
      //fartsaksellerator for de to midtre kontrollpunktene
      //Disse brukes for  f en fin effekt ved blaing.
      double speedFactorMidOut = 0.0;
      double speedFactorMidInn = 0.0;
      
      //Variabler som hvis er true, skal de respektable variablene stoppe  flytte seg. Da har de kommet p plass..
      boolean stopMidInnZ = false;
      boolean stopMidInnX = false;
      boolean stopMidOutZ = false;
      boolean stopMidOutX = false;
      boolean stopOutZ = false;
      boolean stopOutX = false;
      
      //Variabler for  beregne neste verdi til siden.
      float xOuter = myPage.getOutX();
      float zOuter = myPage.getOutZ();
      float xMidOuter = myPage.getMidOutX();
      float zMidOuter = myPage.getMidOutZ();
      float xMidInner = myPage.getMidInnX();
      float zMidInner = myPage.getMidInnZ();
      
      //Stop variabler
      float fstopOutZ;
      if(first < 1){ //Skiller p om det er frste gang siden blir bladd, eller ikke
	 fstopOutZ = 0.0f;
      }else{
	 fstopOutZ = myPage.getLeftd4z() + decFact;
      }
      
      float fstopMidInnZ = myPage.getLeftd2z() + decFact;
      float fstopMidInnX = myPage.getLeftd2x() + decFact*3.5f;
      float fstopMidOutZ = myPage.getLeftd3z() + decFact;
      float fstopMidOutX = myPage.getLeftd3x();
      float fstopOutX = myPage.getLeftd4x() + decFact;
      
      //Variabel for  vite hvor langt vi er fra aksen
      float distOuter = myPage.getOutZ();
      float distMidOut = myPage.getMidOutZ();
      float distMidInn = myPage.getMidInnZ();
      
      float extra = 0.0f;
      
      //Lkke som blir stoppet nr punktene er p plass.
      for(double i = 0;; i++){
	 
	 //Hvis den innerste midtre verdien er p plass, stopper vi
	 if(myPage.getMidInnZ() < lastPageMidInnZ && xOuter < 0.0f){
	    stopMidInnZ = true;
	 }
	 
	 //Hvis den ytterste midtre verdien er p plass, stopper vi.
	 if(myPage.getMidOutZ() < lastPageMidOutZ && xOuter < 0.0f){
	    stopMidOutZ = true;
	    zMidOuter = lastPageMidOutZ;
	 }
	 
	 //Bruker en ekstra-variabel for  kompensere litt p blaingen
	 if(xOuter < 0.0f){
	    if(first<1)extra += decFactLocal*1.5;
	    else extra += decFactLocal*1.3;
	 }
	 
	 //Beregner neste punkt for enden av siden
	 //Skiller p om dette er frste side eller ikke
	 //Hvis vi har kommet til "Stop" slutter vi. Dette er Xverdien vi beregner her.
	 float tmpVarX = (float)(outerRadius * Math.cos(Math.PI * (double)(i*2/100))) - extra;
	 if(first < 1){
	    if(tmpVarX <= fstopOutX - 0.5f){
	       debug("HAR TRUFFET N");
	       xOuter = fstopOutX - 0.5f;
	    }else{
	       xOuter = tmpVarX;
	       debug("HAR IKKE TRUFFET ENDA");
	    }
	 }else{
	    if(tmpVarX <= fstopOutX){
	       debug("HAR TRUFFET N");
	       xOuter = fstopOutX;
	    }else{
	       xOuter = tmpVarX;
	       debug("HAR IKKE TRUFFET ENDA");
	    }
	 }
	 
	 //Beregner neste punkt for enden av siden
	 //Hvis vi har kommet til "Stop" slutter vi. Dette er Zverdien vi beregner her.
	 //Hvis den ytterste z-verdien har kommet p plass, har alle andre verdier ogs
	 //kommet p plass, og vi returnerer fra metoden.
	 float tmpVar = distOuter + (float)(outerRadius * Math.sin(Math.PI * (double)(i*2/100)));
	 if(tmpVar <= fstopOutZ && xOuter < 0.0f){
	    zOuter = fstopOutZ;
	    debug("zOuter: " + zOuter + "\n");
	    myPage.addTurningValues(xOuter, zOuter, xMidOuter, zMidOuter, xMidInner, zMidInner);
	    repaint();
	    return;
	 }else{
	    zOuter = tmpVar;
	 }
	 
	 //Sjekker om vi skal begynne  bevege p det ytterste midtre punktet.
	 //Dette begynner nr det ytre punktets x-verdi er lik eller mindre
	 //enn dette punktets x-verdi. Vi legger til speedfactor her for  f denne
	 //til  komme p plass fr det ytterste punktet.
	 if(xOuter < myPage.getMidOutX() && !stopMidOutZ){
	    speedFactorMidOut += 2.0;
	    zMidOuter = distMidOut + (float)(midOutRadius * Math.sin(Math.PI * (double)(speedFactorMidOut*2/100)));
	    xMidOuter = (float)(midOutRadius * Math.cos(Math.PI * (double)(speedFactorMidOut*2/100)));
	 }
	 

	 
	 //Sjekker om vi skal begynne  bevege det innerste midtre punktet. Dette begynner nr
	 //Det ytterste punktets x-verdi krysser dette punktets x-verdi. Dette punktet drar vi bare
	 //p skr rett ned p plass.
	 if(myPage.getOutX() <= myPage.getMidInnX()){
	    if(myPage.getMidInnX() <= fstopMidInnX){
	       xMidInner = fstopMidInnX;
	    }else{
	       xMidInner -= 0.02f;
	    }
	    
	    if(myPage.getMidInnZ() <= fstopMidInnZ){
	       zMidInner = fstopMidInnZ;
	    }else{
	       zMidInner -= 0.02f;
	    }
	 }
	 
	 //Setter de nye verdiene til myPage, som oppdaterer arrayene.
	 myPage.addTurningValues(xOuter, zOuter, xMidOuter, zMidOuter, xMidInner, zMidInner);
	 repaint();
	 try{
	    Thread.sleep(50);
	 }catch(Exception ex){
	    ex.printStackTrace();
	 }
      }
   }


   /**
    * drawLines
    *
    * Metode som tegner ut de 3 aksene.
    */
   private void drawLines(){
      myColor.setRedMaterial();
      gl.glBegin(GL_LINES);
      gl.glNormal3f(-1.0f,0.0f,0.0f);
      gl.glVertex3f(-3.0f,0.0f,0.0f); //x, y, z
      gl.glVertex3f(3.0f,0.0f,0.0f);
      gl.glEnd();
      
      myColor.setGreenMaterial();
      gl.glBegin(GL_LINES);
      gl.glNormal3f(-1.0f,0.0f,0.0f);
      gl.glVertex3f(0.0f,-3.0f,0.0f); //x, y, z
      gl.glVertex3f(0.0f,3.0f,0.0f);
      gl.glEnd();
      
      myColor.setBlueMaterial();
      gl.glBegin(GL_LINES);
      gl.glNormal3f(-1.0f,0.0f,0.0f);
      gl.glVertex3f(0.0f,0.0f,-3.0f); //x, y, z
      gl.glVertex3f(0.0f,0.0f,3.0f);
      gl.glEnd();
   }
   
   
   /**
    * drawLeftKant
    *
    * Metode som tegner ut ryggen p innholdet
    */
   public void drawLeftKant(){
      myColor.setWhiteMaterial();
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,2,0.0f,1.0f,6,4, myPage.getLeftPages());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      myColor.setLightBlueMaterial();
      
   }
   
   /**
    * drawKanter
    * @param kant Boolsk verdi som sier om det skal vre tekstur p kanten
    * @param side Boolsk verdi som sier om det skal vre tekstur p siden.
    */
   public void drawKanter(boolean kant, boolean side){
      myColor.setWhiteMaterial();
      
      gl.glPushMatrix();
      
      if(kant){
	 myTextureLoader.bind(1);
	 gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	 
	 gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	 gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, myPage.getTxpts());
      }
      
      //Tegner ut de 3 flatene som gjr at det ser ut som om  det er en full bok
      gl.glTranslatef(0.5f, 0.0f, 0.0f);
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getBotPages());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
            
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getTopPages());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);

      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getRightPages());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      if(kant)gl.glDisable(GL_MAP2_TEXTURE_COORD_2);
      
      
      //Tegner ut siden som skal blas
      if(side){
	 myTextureLoader.bind(page);
	 gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	 
	 gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, myPage.getTxpts());
      }
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getTextPage());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      if(side)gl.glDisable(GL_MAP2_TEXTURE_COORD_2);
      
      //Tegner ut siste side i boken
      if(kant){
	 myTextureLoader.bind(2);
	 gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	 gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, myPage.getTxpts());
      }
      gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getLastPage());
      gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
      gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
      if(kant)gl.glDisable(GL_MAP2_TEXTURE_COORD_2);
      
      //Tegner ut neste side, hvis denne skal vises
      if(showNext){
	 int toShow = page;
	 if(page+1 > images.length-1){
	    toShow = 1;
	 }else{
	    toShow++;
	 }
	 
	 if(side){
	    myTextureLoader.bind(toShow);
	    gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	    gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, myPage.getTxpts());
	    gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	 }
	 gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getNextPage());
	 gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
	 gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
	 if(side)gl.glDisable(GL_MAP2_TEXTURE_COORD_2);
      }
      
      //Tegner ut forrige side, hvis denne skal vises
      if(showPrev){
	 if(side){
	    myTextureLoader.bind(page-1);
	    gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	    gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, myPage.getTxpts());
	    gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	 }
	 gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getPrevPage());
	 gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
	 gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
	 if(side)gl.glDisable(GL_MAP2_TEXTURE_COORD_2);
      }
      
      //Tegner ut venstre side av bokens som skal se ut som om det er innhold der.
      if(first > 1){
	 if(kant){
	    myTextureLoader.bind(1);
	    gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	    
	    gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	    gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, myPage.getTxpts());
	 }
	 
	 gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getLeftBotPages());
	 gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
	 gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
	 
	 gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getLeftTopPages());
	 gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
	 gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
	 
	 gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getLeftLeftPages());
	 gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
	 gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
	 if(kant)gl.glDisable(GL_MAP2_TEXTURE_COORD_2);
	 
	 if(kant){
	    myTextureLoader.bind(2);
	    gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	    
	    gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
	    gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, myPage.getTxpts());
	    
	 }
	 
	 //Tegner ut frste side av boken som n ligger underst i bunken p venstre side.
	 gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4, myPage.getLeftLastPage());
	 gl.glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);
	 gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
	 if(kant)gl.glDisable(GL_MAP2_TEXTURE_COORD_2);
      }
      gl.glPopMatrix();
      myColor.setLightBlueMaterial();
   }
   
   
   /**
    * reshape
    * @param w Den nye vidden p canvaset
    * @param h Den nye hyden p canvaset
    *
    * Metode som tar seg av reshaping av canvaset
    */
   public void reshape(int w, int h){
      gl.glViewport(0, 0, w, h);
      gl.glMatrixMode(GL_PROJECTION);
      gl.glLoadIdentity();
      glu.gluPerspective( 45.0, (float) w/h, 1.0, 500.0 );
      gl.glMatrixMode(GL_MODELVIEW);
   }
   
   /**
    * setOpen
    * @param toOpen Bestemmer om openBook, eller turningProcess skal kalles
    *
    * Denne metoden setter et skille om boken skal pnes, eller om
    * en side skal blas.
    */
   public void setOpen(boolean toOpen){
      this.toOpen = toOpen;
   }
   
   /**
    * run
    *
    * Metode som gjr denne klassen til en thread.
    * Hvis boken skal pnes, kalles openBook
    * Hvisk en side skal blas, kalles turningProcess
    * ker ogs antall blainger (first ++);
    */
   public void run(){
      if(toOpen)openBook();
      if(!toOpen){
	 turningProcess();
	 first ++;
      }
      
   }
   
   
   /**
    * navigateUp
    *
    * Metode som brukes i forbindelse med zooming
    */
   private void navigateUp(){
      naviZ += 0.1f;
      this.repaint();
   }
   
   /**
    * navigateDown
    *
    * Metode som brukes i forbindelse med zooming
    */
   private void navigateDown(){
      naviZ += -0.1f;
      this.repaint();
      
   }
   
   /**
    * navigateLeft
    *
    * Metode som brukes i forbindelse med zooming
    */
   private void navigateLeft(){
      naviX += -0.1f;
      this.repaint();
      
   }
   
   /**
    * navigateRight
    *
    * Metode som brukes i forbindelse med zooming
    */
   private void navigateRight(){
      naviX += 0.1f;
      this.repaint();
      
   }
   
   /**
    * keyPressed
    * @param e KeyEvent som sier hvilken tast som ble trykket
    *
    * Metode som sjekker om piltastene ble trykket p, og i s fall
    * kaller p en av zoome-metodene
    */
   public void keyPressed(KeyEvent e){
      int keyCode = e.getKeyCode();
      switch(keyCode){
      case KeyEvent.VK_DOWN:{
	 navigateUp();
	 break;
      }
      case KeyEvent.VK_UP:{
	 navigateDown();
	 break;
      }
      case KeyEvent.VK_LEFT:{
	 navigateLeft();
	 break;
      }
      case KeyEvent.VK_RIGHT:{
	 navigateRight();
	 break;
      }
      }
   }
   public void keyReleased(KeyEvent e){}
   public void keyTyped(KeyEvent e){}
   
   
   /**
    * debug
    * @param s Stringen som skal skrives ut
    *
    * Dette er den standarde debug metoden.
    */
   
   private void debug(String s){
      if(debugOn){
	 System.out.println(s);
      }
   }
}
