/**
 * <strong>Program:</strong>      BildeBok<br>
 * <strong>File:</strong>         Perm.java<br>
 * <strong>Description:</strong>  klasse som holder rede p alle punkter til permen av boken<br>
 * <strong>Author:</strong>       Kenneth Stigen - kenneths@frodo.hiof.no<br>
 * <strong>Environment:</strong>  Windows - jdk 1.3.1<br>
 * <strong>Notes:</strong>        Dette er en del av prosjektet i grafisk databehandling V2002<br>
 *
 * @author Kenneth Stigen (mailto: kenneths@frodo.hiof.no, web: http://frodo.hiof.no/~kenneths)
 * @version Version 1.0
 */
public class Perm {
   
   //PRIVATE VARIABLES
   private boolean debugOn = false;
   
   //Variabler for ryggen av boken (ytre)
   private float xSecBackPerm = 0.5f;
   private float xThirdBackPerm = 0.5f;
   private float zSec1BackPerm = -0.025f;
   private float zSec3BackPerm = 0.47f;
   private float zThird1BackPerm = 0.0f;
   private float zThird3BackPerm = 0.45f;
   
   //Variabler for ryggen av boken (indre)
   private float innerZFirst = 0.0f;
   private float innerZLast = 0.45f;
   private float innerXSec = 0.45f;
   private float innerXThird = 0.45f;
   
   //Variabler for arrayene til kontrollpunktene
   private float topPerm[];
   private float topPermBot[];
   private float topPermFrontSide[];
   private float topPermBackSide[];
   private float backPerm[];
   private float backPermInner[];
   private float backPermCurlBot[];
   private float backPermCurl[];
   
   
   //Kontrollpunkter for det ytre bindeledd mellom flatene som gjr boken tykk
   private float topOuter[] = {
      4.0f, -2.0f, 0.5f,
      4.0f, 0.0f, 0.5f,
      4.0f, 2.0f, 0.5f,
      
      4.25f, -2.25f, 0.47f,
      4.25f, 0.0f, 0.47f,
      4.25f, 2.25f, 0.47f,
      
      4.0f, -2.0f, 0.45f,
      4.0f, 0.0f, 0.45f,
      4.0f, 2.0f, 0.45f
   };
   
   //Kontrollpunkter for bindeleddet p sidene til boken
   private float curlSide[] = {
      0.5f, -2.0f, 0.5f,
      0.5f, -2.25f, 0.47f,
      0.5f, -2.0f, 0.45f,
      
      0.55f, -2.0f, 0.45f,
      0.55f, -2.25f, 0.42f,
      0.55f, -2.0f, 0.40f,
      
      0.6f, -2.0f, 0.5f,
      0.6f, -2.25f, 0.47f,
      0.6f, -2.0f, 0.45f,
   };
   
   
   //Kontrollpunktet for den lille krllen p forsiden av boken
   private float topCurl[] = {
      0.5f, -2.0f,0.5f,
      0.55f,-2.0f,0.45f,
      0.6f, -2.0f,0.5f,
      
      0.5f, -1.0f,0.5f,
      0.55f,-1.0f,0.45f,
      0.6f, -1.0f,0.5f,
      
      0.5f, 1.0f,0.5f,
      0.55f,1.0f,0.45f,
      0.6f, 1.0f,0.5f,
      
      0.5f, 2.0f,0.5f,
      0.55f,2.0f,0.45f,
      0.6f,2.0f,0.5f,
   };
   
   
   //Kontrollpunktet for den lille krllen p baksiden av boken
   private float botCurl[] = {
      0.5f, -2.0f,0.0f,
      0.55f,-2.0f,0.05f,
      0.6f, -2.0f,0.0f,
      
      0.5f, -1.0f,0.0f,
      0.55f,-1.0f,0.05f,
      0.6f, -1.0f,0.0f,
      
      0.5f, 1.0f,0.0f,
      0.55f,1.0f,0.05f,
      0.6f, 1.0f,0.0f,
      
      0.5f, 2.0f,0.0f,
      0.55f,2.0f,0.05f,
      0.6f,2.0f,0.0f,
   };
   
   
   //Kontrollpunkter for textur
   private float txpts[] =  {
      -1.0f, -1.0f,  0.0f, -1.0f,
      -1.0f, 0.0f,    0.0f, 0.0f
   };
   
   /**
    * Constructor
    *
    * Kaller p metoden som initialiserer/oppdaterer kontrollpunktarrayene
    */
   public Perm(){
      initArr();
   }
   
   /**
    * setBackPermInnerValues
    * @param innerZFirst Verdien som skal legges til innerZFirst
    * @param innerZLast Verdien som skal legges til innerZLast
    * @param innerXSec Verdien som skal legges til innerXSec
    * @param innerXThird Verdien som skal legges til innerXThird
    *
    * Denne metoden brukes nr boken pnes.
    * Den forandrer strrelsen p den indre delen av ryggen p boken.
    * Dette gjres for at det skal vre samme mellomrom mellom ytre
    * og indre perm nr boken pnes. Den indre permen m da ke litt
    * i strrelse nr den pnes. Dette kan ligne litt p hvordan en vanlig
    * bok oppfrer seg nr den pnes.
    */
   public void setBackPermInnerValues(float innerZFirst,	float innerZLast, float innerXSec, float innerXThird){
      this.innerZFirst += innerZFirst;
      this.innerZLast += innerZLast;
      this.innerXSec += innerXSec;
      this.innerXThird += innerXThird;
      initArr();
   }

   /**
    * setBackPermCurl
    * @param xSecBackPerm
    * @param xThirdBackPerm
    * @param zSec1BackPerm
    * @param zSec3BackPerm
    * @param zThird1BackPerm
    * @param zThird3BackPerm
    *
    * Denne metoden endrer bindeleddet mellom ytre og indre del av ryggen.
    * Dette gjres fordi den indre delen forandres, og bindeleddet m fortsatt
    * vre satt opp p riktig mte.
    */
   public void setBackPermCurl(float xSecBackPerm, float xThirdBackPerm, float zSec1BackPerm, float zSec3BackPerm, float zThird1BackPerm, float zThird3BackPerm){
      this.xSecBackPerm += xSecBackPerm ;
      this.xThirdBackPerm += xThirdBackPerm;
      this.zSec1BackPerm += zSec1BackPerm;
      this.zSec3BackPerm += zSec3BackPerm;
      this.zThird1BackPerm += zThird1BackPerm;
      this.zThird3BackPerm += zThird3BackPerm;
   }
   
   /**
    * initArr
    *
    * Metode som oppdaterer/initialiserer alle arrayene med kontrollpunkter.
    */
   private void initArr(){
      
      float tempTopPerm[] = {
         0.6f, -2.0f,0.5f,  4.0f,-2.0f,0.5f,
         0.6f, -1.0f,0.5f,  4.0f,-1.0f,0.5f,
         0.6f, 1.0f,0.5f,  4.0f,1.0f,0.5f,
         0.6f, 2.0f,0.5f,  4.0f,2.0f,0.5f,
	 
      };
      
      
      float tempBotPerm[] = {
         0.6f, -2.0f,0.0f,  4.0f,-2.0f,0.0f,
         0.6f, -1.0f,0.0f,  4.0f,-1.0f,0.0f,
         0.6f, 1.0f,0.0f,  4.0f,1.0f,0.0f,
         0.6f, 2.0f,0.0f,  4.0f,2.0f,0.0f,
	 
      };
      
      
      
      float tempTopPermFrontSide[] = {
         0.6f, -2.0f, 0.5f,    4.0f, -2.0f, 0.5f,
         0.6f, -2.25f, 0.47f,  4.0f + 0.25f, -2.25f, 0.47f,
         0.6f, -2.0f, 0.45f,   4.0f, -2.0f, 0.45f
      };
      
      float tempTopPermBackSide[] = {
         0.6f, 2.0f, 0.5f,    4.0f, 2.0f, 0.5f,
         0.6f, 2.25f, 0.47f,  4.0f + 0.25f, 2.25f, 0.47f,
         0.6f, 2.0f, 0.45f,   4.0f, 2.0f, 0.45f
      };
      
      float tempBotPermFrontSide[] = {
         0.6f, -2.0f, 0.0f,    4.0f, -2.0f, 0.0f,
         0.6f, -2.25f, -0.03f,  4.0f + 0.25f, -2.25f, -0.03f,
         0.6f, -2.0f, -0.05f,   4.0f, -2.0f, -0.05f
      };
      
      float tempBotPermBackSide[] = {
         0.6f, 2.0f, 0.0f,    4.0f, 2.0f, 0.0f,
         0.6f, 2.25f, -0.03f,  4.0f + 0.25f, 2.25f, -0.03f,
         0.6f, 2.0f, -0.05f,   4.0f, 2.0f, -0.05f
      };
      
      
      
      float tempBackPerm[] = {
         0.5f, -2.0f,-0.05f,  0.4f,-2.0f,0.15f,
         0.4f, -2.0f,0.3f,  0.5f,-2.0f,0.5f,
	 
         0.5f, -1.0f,-0.05f,  0.4f,-1.0f,0.15f,
         0.4f, -1.0f,0.3f,  0.5f,-1.0f,0.5f,
	 
         0.5f, 1.0f,-0.05f,  0.4f,1.0f,0.15f,
         0.4f, 1.0f,0.3f,  0.5f,1.0f,0.5f,
	 
         0.5f, 2.0f,-0.05f,  0.4f,2.0f,0.15f,
         0.4f,2.0f,0.3f,  0.5f,2.0f,0.5f,
      };
      
      
      
      float tempBackPermInner[] = {
	 0.5f, -2.0f,innerZFirst,  innerXSec,-2.0f,0.15f,
	 innerXThird, -2.0f,0.3f,  0.5f,-2.0f,innerZLast,
	 
	 0.5f, -1.0f,innerZFirst,  innerXSec,-1.0f,0.15f,
	 innerXThird, -1.0f,0.3f,  0.5f,-1.0f,innerZLast,
	 
	 0.5f, 1.0f,innerZFirst,  innerXSec,1.0f,0.15f,
	 innerXThird, 1.0f,0.3f,  0.5f,1.0f,innerZLast,
	 
	 0.5f, 2.0f,innerZFirst,  innerXSec,2.0f,0.15f,
	 innerXThird,2.0f,0.3f,  0.5f,2.0f,innerZLast,
      };
      
      float tempBackPermCurlBot[] ={
	 0.5f, -2.0f, -0.05f,
	 xSecBackPerm, -2.25f, zSec1BackPerm,
	 xThirdBackPerm, -2.0f, zThird1BackPerm,
	 
	 0.4f, -2.0f, 0.15f,
	 0.42f, -2.25f, 0.15f,
	 0.45f, -2.0f, 0.15f,

	 0.4f, -2.0f, 0.3f,
	 0.42f, -2.25f, 0.3f,
	 0.45f, -2.0f, 0.3f,
	 
	 0.5f, -2.0f, 0.5f,
	 xSecBackPerm, -2.25f, zSec3BackPerm,
	 xThirdBackPerm, -2.0f, zThird3BackPerm,
	 
      };
      
      float tempBackPermCurl[] ={
	 0.5f, 2.0f, -0.05f,
	 xSecBackPerm, 2.25f, zSec1BackPerm,
	 xThirdBackPerm, 2.0f, zThird1BackPerm,
	 
	 0.4f, 2.0f, 0.15f,
	 0.42f, 2.25f, 0.15f,
	 0.45f, 2.0f, 0.15f,
	 
	 0.4f, 2.0f, 0.3f,
	 0.42f, 2.25f, 0.3f,
	 0.45f, 2.0f, 0.3f,
	 
	 0.5f, 2.0f, 0.5f,
	 xSecBackPerm, 2.25f, zSec3BackPerm,
	 xThirdBackPerm, 2.0f, zThird3BackPerm,
	 
      };
      
      
      backPermCurl = tempBackPermCurl;
      backPermCurlBot = tempBackPermCurlBot;
      backPerm = tempBackPerm;
      backPermInner = tempBackPermInner;
      topPerm = tempTopPerm;
      topPermFrontSide = tempTopPermFrontSide;
      topPermBackSide = tempTopPermBackSide;
      
   }
   
   
   /**
    * Nedenfor flger divers getmetoder.
    * Jeg ser ikke behovet for  kommentere alle disse
    * Alle disse returnerer en array med kontrollpunkter.
    */
   
   
   public float[] getTxpts(){
      return this.txpts;
   }
   public float[] getTopPerm(){
      return this.topPerm;
   }
   
   public float[] getTopPermFrontSide(){
      return this.topPermFrontSide;
   }
   
   public float[] getTopPermBackSide(){
      return this.topPermBackSide;
   }
   
   public float[] getBackPerm(){
      return this.backPerm;
   }
   public float[] getBackPermInner(){
      return this.backPermInner;
   }
   public float[] getBackPermCurl(){
      return this.backPermCurl;
   }
   public float[] getBackPermCurlBot(){
      return this.backPermCurlBot;
   }
   
   public float[] getTopCurl(){
      return this.topCurl;
   }
   
   public float[] getBotCurl(){
      return this.botCurl;
   }
   
   public float[] getCurlSide(){
      return this.curlSide;
   }
   
   public float[] getTopOuter(){
      return this.topOuter;
   }
   
   
   /**
    * debug
    * @param s Stringen som skal skrives ut
    *
    * Dette er den standarde debug metoden.
    */
   private void debug(String s){
      if(debugOn){
	 System.out.println(this.getClass().getName() + " : " + s);
      }
   }
}
